==> fsframework-extend-1.2.9.46-sources.jar: com/fshows/fsframework/extend/dubbo/loadbalance/StableLoadBalance.java <==
/**
 * fshows.com
 * Copyright (C) 2013-2019 All Rights Reserved.
 */
package com.fshows.fsframework.extend.dubbo.loadbalance;

import cn.hutool.core.util.StrUtil;
import com.alibaba.dubbo.common.URL;
import com.alibaba.dubbo.rpc.Invocation;
import com.alibaba.dubbo.rpc.Invoker;
import com.alibaba.dubbo.rpc.RpcContext;
import com.alibaba.dubbo.rpc.cluster.loadbalance.RandomLoadBalance;
import com.alibaba.fastjson.JSON;
import com.beust.jcommander.internal.Lists;
import com.fshows.fsframework.core.constants.StringPool;
import com.fshows.fsframework.core.utils.LogUtil;
import lombok.extern.slf4j.Slf4j;

import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * @author buhao
 * @version SitLoadBalance.java, v 0.1 2019-11-26 21:57 buhao
 */
@Slf4j
public class StableLoadBalance extends RandomLoadBalance {

    public static final String NAME = "stable";

    /**
     * 上下文中，路由 tab 的 key
     */
    public static final String ROUTE_TAG_CONTEXT_KEY = "ROUTE_TAG_CONTEXT_KEY";
    /**
     * stable 环境路由 tab key
     */
    public static final String STABLE_ROUTE_TAG_KEY = "stable.route.tag";
    /**
     * 默认路由 tab key
     */
    public static final String DEFAULT_ROUTE_TAG_KEY = "dubbo.provider.parameters.route";
    /**
     * dubbo 中默认保存的路由标签 key
     */
    private static final String DUBBO_DEFAULT_ROUTE_TAG_KEY = "default.route";
    /**
     * dubbo 中保存的路由标签 key
     */
    private static final String DUBBO_ROUTE_TAG_KEY = "route";

    @Override
    public <T> Invoker<T> select(List<Invoker<T>> invokers, URL url, Invocation invocation) {
        return doSelect(invokers, url, invocation);
    }

    @Override
    protected <T> Invoker<T> doSelect(List<Invoker<T>> invokers, URL url, Invocation invocation) {
        LogUtil.debug(log, "doSelect >> 路由过滤开始 >> invokers = {}, invocation = {}", JSON.toJSONString(invokers), invocation);
        // 获得路由 TAG 根据上下文
        String sourceOwnerByContext = getRouteTagByContext();
        // 根据上下文中路由 TAG 过滤到的可调用节点
        List<Invoker<T>> filterInvokersByGroup = filterInvokersByContextRouteTag(invokers, sourceOwnerByContext);
        // 如果都为空取 stable 节点
        if (StrUtil.isBlank(sourceOwnerByContext) || filterInvokersByGroup.isEmpty()) {
            filterInvokersByGroup = filterInvokersByStableRouteTag(invokers);
        }
        // 如果过滤结果不为空，覆盖结果
        if (!filterInvokersByGroup.isEmpty()) {
            invokers = filterInvokersByGroup;
        }
        LogUtil.debug(log, "doSelect >> 路由过滤结束 >> invokers = {}, invocation = {}", JSON.toJSONString(invokers), invocation);
        return super.doSelect(invokers, url, invocation);
    }

    /**
     * 根据 stable 路由标签过滤调用列表
     *
     * @param invokers
     * @param <T>
     * @return
     */
    private <T> List<Invoker<T>> filterInvokersByStableRouteTag(List<Invoker<T>> invokers) {
        final List<Invoker<T>> result = invokers.stream().filter(this::isStableRouteTag).collect(Collectors.toList());
        LogUtil.debug(log, "filterInvokersByStableRouteTag >> 根据 stable route tag 过滤 >> invokers = {}", JSON.toJSONString(invokers));
        return result;
    }

    /**
     * 是否为 stable 环境的路由标签
     *
     * @param it
     * @param <T>
     * @return
     */
    private <T> boolean isStableRouteTag(Invoker<T> it) {
        return isMatchExpectRouteTag(it, System.getProperty(STABLE_ROUTE_TAG_KEY));
    }

    /**
     * 判断是否符合期望的路由标签
     *
     * @param it
     * @param expectRouteTag
     * @param <T>
     * @return
     */
    private <T> boolean isMatchExpectRouteTag(Invoker<T> it, String expectRouteTag) {
        return StrUtil.equalsIgnoreCase(getProviderRouteTagByDubboInvoker(it), expectRouteTag);
    }

    /**
     * 根据上下文路由标签过滤调用列表
     *
     * @param invokers
     * @param sourceOwnerByContext
     * @param <T>
     * @return
     */
    private <T> List<Invoker<T>> filterInvokersByContextRouteTag(List<Invoker<T>> invokers, String sourceOwnerByContext) {
        // 获得源 owner 节点过滤
        List<Invoker<T>> result = Lists.newArrayList();
        if (StrUtil.isNotBlank(sourceOwnerByContext)) {
            result = invokers.stream().filter(it -> isMatchExpectRouteTag(it, sourceOwnerByContext)).collect(Collectors.toList());
        }
        LogUtil.debug(log, "doSelect >> 根据上下文路由 TAG 过滤调用列表 >> sourceOwnerByContext = {}， result = {}",
                JSON.toJSONString(sourceOwnerByContext), JSON.toJSONString(result));
        return result;
    }

    /**
     * 从 dubbo 的 invoker 取出 Provider 的 Route Tag
     *
     * @param invoker
     * @param <T>
     * @return
     */
    private <T> String getProviderRouteTagByDubboInvoker(Invoker<T> invoker) {
        // 如果 Invoker 为空就返回空串
        if (Objects.isNull(invoker)) {
            return StringPool.EMPTY;
        }
        // 如果 url 为空就返回空串
        final URL url = invoker.getUrl();
        if (Objects.isNull(url)) {
            return StringPool.EMPTY;
        }
        // 如果服务路由标签不为空就直接返回
        final String routeTag = url.getParameter(DUBBO_ROUTE_TAG_KEY);
        if (StrUtil.isNotBlank(routeTag)) {
            return routeTag;
        }
        // 返回默认标签
        return url.getParameter(DUBBO_DEFAULT_ROUTE_TAG_KEY, StringPool.EMPTY);
    }

    /**
     * 获得上下文中的路由标签
     *
     * @return
     */
    private String getRouteTagByContext() {
        // 获得上下文路由标签
        String result = RpcContext.getContext().getAttachment(ROUTE_TAG_CONTEXT_KEY);
        LogUtil.debug(log, "doSelect >> 获得上下文路由 TAG >> result = {}", result);
        // 如果上下文没有源路由标签，就从环境变量取默认标签
        if (StrUtil.isBlank(result)) {
            result = System.getProperty(DEFAULT_ROUTE_TAG_KEY);
            if (StrUtil.isNotBlank(result)) {
                // 将默认路由标签存入上下文
                RpcContext.getContext().setAttachment(ROUTE_TAG_CONTEXT_KEY, result);
                LogUtil.debug(log, "doSelect >> 上下文路由 TAG 为空,从环境变量中取，并设置到上下文中 >> result = {}", result);
            }
        }
        return result;
    }
}

==> fsframework-extend-2.0.2-20200424.070935-1-sources.jar: com/fshows/fsframework/extend/dubbo/loadbalance/StableLoadBalance.java <==
/**
 * fshows.com
 * Copyright (C) 2013-2019 All Rights Reserved.
 */
package com.fshows.fsframework.extend.dubbo.loadbalance;

import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSON;
import com.beust.jcommander.internal.Lists;
import com.fshows.fsframework.core.utils.LogUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.common.URL;
import org.apache.dubbo.rpc.Invocation;
import org.apache.dubbo.rpc.Invoker;
import org.apache.dubbo.rpc.RpcContext;
import org.apache.dubbo.rpc.cluster.loadbalance.RandomLoadBalance;

import java.util.List;
import java.util.stream.Collectors;

/**
 * @author buhao
 * @version SitLoadBalance.java, v 0.1 2019-11-26 21:57 buhao
 */
@Slf4j
public class StableLoadBalance extends RandomLoadBalance {

    public static final String NAME = "stable";

    /**
     * 上下文中，路由 tab 的 key
     */
    public static final String ROUTE_TAG_CONTEXT_KEY = "ROUTE_TAG_CONTEXT_KEY";
    /**
     * stable 环境路由 tab key
     */
    public static final String STABLE_ROUTE_TAG_KEY = "stable.route.tag";
    /**
     * 默认路由 tab key
     */
    public static final String DEFAULT_ROUTE_TAG_KEY = "dubbo.provider.owner";
    /**
     * dubbo 中保存的路由标签 key
     */
    private static final String DUBBO_ROUTE_TAG_KEY = "default.owner";

    @Override
    public <T> Invoker<T> select(List<Invoker<T>> invokers, URL url, Invocation invocation) {
        return doSelect(invokers, url, invocation);
    }

    @Override
    protected <T> Invoker<T> doSelect(List<Invoker<T>> invokers, URL url, Invocation invocation) {
        LogUtil.info(log, "doSelect >> 路由过滤开始 >> invokers = {}, invocation = {}", JSON.toJSONString(invokers), invocation);
        // 获得路由 TAG 根据上下文
        String sourceOwnerByContext = getRouteTagByContext();
        // 根据上下文中路由 TAG 过滤到的可调用节点
        List<Invoker<T>> filterInvokersByGroup = filterInvokersByContextRouteTag(invokers, sourceOwnerByContext);
        // 如果都为空取 stable 节点
        if (StrUtil.isBlank(sourceOwnerByContext) || filterInvokersByGroup.isEmpty()) {
            filterInvokersByGroup = filterInvokersByStableRouteTag(invokers);
        }
        // 如果过滤结果不为空，覆盖结果
        if (!filterInvokersByGroup.isEmpty()) {
            invokers = filterInvokersByGroup;
        }
        LogUtil.info(log, "doSelect >> 路由过滤结束 >> invokers = {}, invocation = {}", JSON.toJSONString(invokers), invocation);
        return super.doSelect(invokers, url, invocation);
    }

    /**
     * 根据 stable 路由标签过滤调用列表
     *
     * @param invokers
     * @param <T>
     * @return
     */
    private <T> List<Invoker<T>> filterInvokersByStableRouteTag(List<Invoker<T>> invokers) {
        final List<Invoker<T>> result = invokers.stream().filter(it -> StrUtil.equalsIgnoreCase(it.getUrl().getParameter(DUBBO_ROUTE_TAG_KEY), System.getProperty(STABLE_ROUTE_TAG_KEY))).collect(Collectors.toList());
        LogUtil.info(log, "filterInvokersByStableRouteTag >> 根据 stable route tag 过滤 >> invokers = {}", JSON.toJSONString(invokers));
        return result;
    }

    /**
     * 根据上下文路由标签过滤调用列表
     *
     * @param invokers
     * @param sourceOwnerByContext
     * @param <T>
     * @return
     */
    private <T> List<Invoker<T>> filterInvokersByContextRouteTag(List<Invoker<T>> invokers, String sourceOwnerByContext) {
        // 获得源 owner 节点过滤
        List<Invoker<T>> result = Lists.newArrayList();
        if (StrUtil.isNotBlank(sourceOwnerByContext)) {
            result = invokers.stream().filter(it -> StrUtil.equalsIgnoreCase(it.getUrl().getParameter(DUBBO_ROUTE_TAG_KEY), sourceOwnerByContext)).collect(Collectors.toList());
        }
        LogUtil.info(log, "doSelect >> 根据上下文路由 TAG 过滤调用列表 >> sourceOwnerByContext = {}， result = {}",
                JSON.toJSONString(sourceOwnerByContext), JSON.toJSONString(result));
        return result;
    }

    /**
     * 获得上下文中的路由标签
     *
     * @return
     */
    private String getRouteTagByContext() {
        // 获得上下文路由标签
        String result = RpcContext.getContext().getAttachment(ROUTE_TAG_CONTEXT_KEY);
        LogUtil.info(log, "doSelect >> 获得上下文路由 TAG >> result = {}", result);
        // 如果上下文没有源路由标签，就从环境变量取默认标签
        if (StrUtil.isBlank(result)) {
            result = System.getProperty(DEFAULT_ROUTE_TAG_KEY);
            if (StrUtil.isNotBlank(result)) {
                // 将默认路由标签存入上下文
                RpcContext.getContext().setAttachment(ROUTE_TAG_CONTEXT_KEY, result);
                LogUtil.info(log, "doSelect >> 上下文路由 TAG 为空,从环境变量中取，并设置到上下文中 >> result = {}", result);
            }
        }
        return result;
    }
}
